package org.dbms.service;

import org.dbms.dto.DriverDTO;
import org.dbms.models.Driver;
import org.dbms.searchModels.DriverSearch;
import org.dbms.storageContracts.IDriverStorage;
import org.dbms.storageImpl.DriverStorage;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
public class DriverService {
    private IDriverStorage driverStorage;

    public DriverService(IDriverStorage driverStorage) {
        this.driverStorage = driverStorage;
    }

    @Transactional
    public List<Driver> getFiltered(int expFrom, int expTo) {
        DriverSearch ds = new DriverSearch();

        if(expFrom != -1) ds.expirienceFrom = expFrom;
        if(expTo != -1) ds.expirienceTo = expTo;

        return driverStorage.getFilteredList(ds);
    }

    @Transactional
    public int getFilteredCount(int expFrom, int expTo, int size) {
        DriverSearch ds = new DriverSearch();

        if(expFrom != -1) ds.expirienceFrom = expFrom;
        if(expTo != -1) ds.expirienceTo = expTo;

        int count = driverStorage.getFilteredCount(ds);
        int res = count / size;
        if(count % size != 0) res++;
        return res;
    }
    @Transactional
    public List<Driver> getFilteredPage(int expFrom, int expTo, int size, int page) {
        DriverSearch ds = new DriverSearch();

        if(expFrom != -1) ds.expirienceFrom = expFrom;
        if(expTo != -1) ds.expirienceTo = expTo;

        return driverStorage.getFilteredPage(ds, size, page);
    }

    @Transactional
    public Driver getDriverById(Long driverId) {
        return driverStorage.getElementById(driverId);
    }
}
